@@ -1,8 +1,7 @@
 /*
  * wmnd - window maker network devices - drivers.c
- *
- * drivers definitions
- * $Id: drivers.c,v 1.10 2003/05/15 15:44:11 wavexx Exp $
+ * drivers definitions - implementation
+ * Copyright(c) 2001-2004 by wave++ "Yuri D'Elia" <wavexx@users.sf.net>
  */
 
 /* local headers */
@@ -387,19 +386,15 @@ solaris_kstat_term(struct Devices* dev)
 int
 testing_dummy_list(const char* devname, struct Devices* list)
 {
-  char* devn;
   struct Devices* ndev;
-
-  if(!devname)
-    devn = strdup("off");
-  else
-    devn = strdup(devname);
+  if(devname)
+    return 0;
 
   ndev = (struct Devices*)malloc(sizeof(struct Devices));
-  ndev->name = devn;
+  ndev->name = strdup("off");
   devices_append(list, ndev);
 
-  msg_drInfo(drName, "detected %s", devn);
+  msg_drInfo(drName, "detected %s", ndev->name);
 
   return 1; /* usually returns the number of devices */
 }
@@ -466,13 +461,18 @@ linux_proc_list(const char* devname, struct Devices* list)
   /* device name was specified */
   if(devname)
   {
-    dta = 1;
-    ndev = (struct Devices*)malloc(sizeof(struct Devices));
-    ndev->devstart = 0;
-    ndev->name = strdup(devname);
-    devices_append(list, ndev);
+    strncpy(temp, devname, sizeof(temp));
+    for(p = strtok(temp, ","); p; p = strtok(NULL, ","))
+    {
+      ndev = (struct Devices*)malloc(sizeof(struct Devices));
+      ndev->devstart = 0;
+      ndev->name = strdup(p);
 
-    msg_drInfo(drName, "forced %s", devname);
+      ++dta;
+      list = devices_append(list, ndev);
+
+      msg_drInfo(drName, "forced %s", p);
+    }
   }
   else
   {
@@ -524,6 +524,8 @@ linux_proc_get(struct Devices* dev, unsigned long* ip, unsigned long* op,
 
   /* read statistics from network device's list */
   fp = fopen(linux_proc_netDevice, "r");
+  if(!fp) return 1;
+
   fgets(temp, MAXBUF, fp);
   fgets(temp, MAXBUF, fp);
 
@@ -780,9 +782,10 @@ irix_pcp_list(const char* devname, struct Devices* list)
         (!devname && strcmp(*p, "lo0")))
     {
       msg_drInfo(drName, "detected %s(%d)", *p, *t);
-      ndev = malloc(sizeof(struct Devices));
+      ndev = (struct Devices*)malloc(sizeof(struct Devices));
       ndev->name = strdup(*p);
-      drdata = malloc(sizeof(struct irix_pcp_drvdata));
+      drdata = (struct irix_pcp_drvdata*)malloc(
+	  sizeof(struct irix_pcp_drvdata));
       memcpy(drdata->pmId, pmId, sizeof(pmId));
       drdata->inst = *t;
       ndev->drvdata = (void*)drdata;
@@ -872,6 +875,7 @@ irix_pcp_term(struct Devices* dev)
 #define SNMPMIB_COMN "public"
 #define SNMPMIB_NUM "ifNumber.0"
 #define SNMPMIB_STATUS "ifOperStatus"
+#define SNMPMIB_CHANGE "ifLastChange"
 #define SNMPMIB_IFINBDEF "ifInOctets"
 #define SNMPMIB_IFINPDEF "ifInUcastPkts"
 #define SNMPMIB_IFOUTBDEF "ifOutOctets"
@@ -880,12 +884,15 @@ irix_pcp_term(struct Devices* dev)
 /*
  * In WMND 0.4.5 I fetch ifName instead of ifDescr (ifName is shorter). I
  * noticed however that ifName is buggy on some firmware vendors (DLINK) and
- * there's no way to get around the bug without causing a network flood.
+ * there's no way to get around the bug without causing a network flood, so
+ * simply invert the order.
  */
 #ifndef USE_GENERIC_SNMP_DESCR
-#define SNMPMIB_NAME "ifName"
+#define SNMPMIB_NAME_A "ifName"
+#define SNMPMIB_NAME_B "ifDescr"
 #else
-#define SNMPMIB_NAME "ifDescr"
+#define SNMPMIB_NAME_A "ifDescr"
+#define SNMPMIB_NAME_B "ifName"
 #endif
 
 struct generic_snmp_drvdata
@@ -996,13 +1003,13 @@ generic_snmp_comp(const char* name, const int num)
 }
 
 char*
-generic_snmp_getDesc(struct snmp_session* se, int dev)
+generic_snmp_getNodeDesc(struct snmp_session* se, const char* node, int dev)
 {
   struct snmp_pdu* pdu;
   struct snmp_pdu* res;
   oid OID[MAX_OID_LEN];
   size_t OID_len = MAX_OID_LEN;
-  char* name = generic_snmp_comp(SNMPMIB_NAME, dev);
+  char* name = generic_snmp_comp(node, dev);
 
   pdu = snmp_pdu_create(SNMP_MSG_GET);
   get_node(name, OID, &OID_len);
@@ -1023,6 +1030,15 @@ generic_snmp_getDesc(struct snmp_session* se, int dev)
   return name;
 }
 
+char*
+generic_snmp_getDesc(struct snmp_session* se, int dev)
+{
+  char* desc;
+
+  return ((desc = generic_snmp_getNodeDesc(se, SNMPMIB_NAME_A, dev))? desc:
+      generic_snmp_getNodeDesc(se, SNMPMIB_NAME_B, dev));
+}
+
 struct Devices*
 generic_snmp_preInit(struct Devices* list, struct snmp_session* se, int dev)
 {
