@@ -15,8 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- * $Id: wmnd.c,v 1.23 2003/06/04 07:48:32 wavexx Exp $
  */
 
 #include "wmnd.h"
@@ -45,7 +43,6 @@ void new_window(char* res_name, char* res_class,
 void conf_read(char* filename);
 void conf_write(char* filename);
 void assign(char* name, char* value);
-void vdel(char* name);
 struct var *lookup(char* name);
 char* value(char* name);
 char* vcopy(char* str);
@@ -77,6 +74,7 @@ void draw_stats(struct Devices *ptr, const int gap);
 /* driver functions */
 int devices_init(const char* driver, const char* interface);
 void devices_select(const char* interface);
+void devices_prev(void);
 void devices_getstat(struct Devices *device, unsigned long* ip,
     unsigned long* op, unsigned long* ib, unsigned long* ob);
 void devices_destroy(void);
@@ -113,7 +111,7 @@ check_mr(int x, int y)
       found = 1;
   }
   if(!found)
-    return -1;
+    return REG_NOREG;
   return (i - 1);
 }
 
@@ -155,6 +153,7 @@ void
 new_window(char* res_name, char* res_class,
     int width, int height, int argc, char** argv)
 {
+  char* geometry;
   XpmAttributes attr;
   XpmColorSymbol cols[3] =
   {
@@ -173,9 +172,17 @@ new_window(char* res_name, char* res_class,
   dockapp.screen = DefaultScreen(dockapp.d);
   dockapp.root = DefaultRootWindow(dockapp.d);
 
-  sizehints.flags = USSize | USPosition;
-  sizehints.x = dockapp.x ? dockapp.x : 0;
-  sizehints.y = dockapp.y ? dockapp.y : 0;
+  /* parse the geometry spec, if any */
+  geometry = value("geometry");
+  if(geometry)
+  {
+    unsigned dummy;
+    XParseGeometry(geometry, &dockapp.x, &dockapp.y, &dummy, &dummy);
+  }
+
+  sizehints.flags = USSize | (geometry? USPosition: 0);
+  sizehints.x = dockapp.x;
+  sizehints.y = dockapp.y;
   sizehints.width = width;
   sizehints.height = height;
 
@@ -192,13 +199,11 @@ new_window(char* res_name, char* res_class,
   classhint.res_class = res_class;
   XSetClassHint(dockapp.d, dockapp.win, &classhint);
 
-  XSelectInput(dockapp.d, dockapp.win,
-      ExposureMask | ButtonPressMask | ButtonReleaseMask |
-      StructureNotifyMask);
-  XSelectInput(dockapp.d, dockapp.iconwin,
-      ExposureMask | ButtonPressMask | ButtonReleaseMask |
-      StructureNotifyMask);
-
+#define EVENTS (ExposureMask | ButtonPressMask | \
+	ButtonReleaseMask | StructureNotifyMask)
+  XSelectInput(dockapp.d, dockapp.win, EVENTS);
+  XSelectInput(dockapp.d, dockapp.iconwin, EVENTS);
+  
   XStoreName(dockapp.d, dockapp.win, res_name);
   XSetIconName(dockapp.d, dockapp.win, res_name);
 
@@ -312,6 +317,7 @@ conf_write(char* filename)
 {
   FILE *fp;
   struct var *vp;
+  int dc;
 
   fp = fopen(filename, "w");
   if(!fp)
@@ -320,8 +326,18 @@ conf_write(char* filename)
     exit(1);
   }
   fprintf(fp, "# WMND configuration file (generated automatically)\n\n");
+
+  /* write only changed values */
   for(vp = vars; vp; vp = vp->v_next)
-    fprintf(fp, "%s=%s\n", vp->v_name, vp->v_value);
+  {
+    if(!vp->v_value)
+      continue;
+
+    dc = defcon_lk(vp->v_name);
+    if(dc == -1 || !pss_defcon[dc].value ||
+	strcmp(pss_defcon[dc].value, vp->v_value))
+      fprintf(fp, "%s=%s\n", vp->v_name, vp->v_value);
+  }
   fclose(fp);
 }
 
@@ -329,70 +345,43 @@ conf_write(char* filename)
 void
 assign(char* name, char* value)
 {
-  struct var *vp;
-  struct var *newv;
+  struct var* vp;
+  struct var* newv;
 
-  newv = (struct var *) malloc(sizeof(struct var));
-
-  vdel(name);
-
-  if(vars == NULL || strcmp(vars->v_name, name) >= 0)
+  /* search for existing values */
+  for(vp = vars; vp &&
+	(vp->v_next || !strcmp(name, vp->v_name)); vp = vp->v_next)
   {
-    newv->v_next = vars;
-    vars = newv;
-  }
-  else
-  {
-    vp = vars;
-    while (vp->v_next && strcmp(vp->v_next->v_name, name) <= 0)
-      vp = vp->v_next;
-    newv->v_next = vp->v_next;
-    vp->v_next = newv;
+    if(!vp->v_next || !strcmp(name, vp->v_name))
+    {
+      /* existing value */
+      if(vp->v_value)
+	free(vp->v_value);
+      vp->v_value = vcopy(value);
+      return;
+    }
   }
+   
+  /* append the value */
+  newv = (struct var*)malloc(sizeof(struct var));
   newv->v_name = vcopy(name);
   newv->v_value = vcopy(value);
-}
+  newv->v_next = NULL;
 
-
-void
-vdel(char* name)
-{
-  struct var *vp;
-  struct var *newv;
-
-  while (vars && !strcmp(vars->v_name, name))
-  {
-    vp = vars->v_next;
-    free((struct var *) vars);
-    vars = vp;
-  }
-  if(vars)
-  {
-    for(newv = vars; newv->v_next;)
-      if(!strcmp(newv->v_next->v_name, name))
-      {
-        vp = newv->v_next->v_next;
-        free(newv->v_next);
-        newv->v_next = vp;
-      }
-      else
-        newv = newv->v_next;
-  }
+  if(!vars)
+    vars = newv;
+  else
+    vp->v_next = newv;
 }
 
 
 char*
 vcopy(char* str)
 {
-  char* newv;
-  unsigned int len;
-
   if(str == NULL)
-    return "";
-  len = strlen(str) + 1;
-  newv = (char*)malloc(len);
-  memcpy(newv, str, len);
-  return newv;
+    return NULL;
+
+  return strdup(str);
 }
 
 
@@ -427,6 +416,7 @@ waveval_fe(const struct drwStruct *data, const char* val)
 }
 
 
+/* Lookup for an index in the defcon table */
 int
 defcon_lk(const char* token)
 {
@@ -441,6 +431,7 @@ defcon_lk(const char* token)
 }
 
 
+/* Touch an item in the defcon table (mark as 'used') */
 void
 defcon_touch(char* token, char* val)
 {
@@ -510,24 +501,143 @@ beat_event(void)
 }
 
 
+void
+exec_perc_command(const char* cmd, int button)
+{
+  char buf[4];
+  char* newcmd;
+  perctbl_t data[3];
+
+  /* button */
+  sprintf(buf, "%d", button);
+  data[0].c = 'b';
+  data[0].value = buf;
+
+  /* interface name */
+  data[1].c = 'i';
+  data[1].value = wmnd.curdev->name;
+
+  /* status */
+  data[2].c = 's';
+  data[2].value = (wmnd.curdev->online? "0": "1");
+
+  /* parse and exec the new command */
+  newcmd = percsubst(cmd, data, 3);
+  exec_command(newcmd);
+  free(newcmd);
+}
+
+
+#ifdef USE_TREND
+const char*
+tmpfifo()
+{
+  const char* file;
+  int retries = 26;
+
+  while(retries--)
+  {
+    /* Nouuuuuu (sound of a lemming exploding)!
+     * Don't use THAT function! IT'S BAD! BAD BAAAD! Will kill 'ya!
+     * MAVAFF... I'd use tmpfile _anyway_ */
+    file = tmpnam(NULL);
+    if(!file) return NULL;
+    if(!mkfifo(file, 0600))
+      return file;
+  }
+
+  return NULL;
+}
+
+
+void
+exec_trend(int rt, int bp)
+{
+  int idx, i;
+  const char* title;
+  const char* file;
+  FILE* fd;
+
+  idx = (bp? 0: 2);	/* bytes or packets */
+  idx += (rt? 0: 1);	/* input or output */
+
+  /* title */
+  switch(idx)
+  {
+  case 0: title = "input bytes"; break;
+  case 1: title = "ouput bytes"; break;
+  case 2: title = "input packets"; break;
+  case 3: title = "output packets"; break;
+  }
+
+  /* create the fifo */
+  file = tmpfifo();
+  if(!file)
+  {
+    msg_err("cannot create temporary fifo");
+    return;
+  }
+
+  /* execute */
+  switch(fork())
+  {
+  case -1:
+    msg_err("cannot fork!");
+    break;
+
+  case 0:
+    execlp("trend", "trend", "-t", title, "-G", (bp? "1000": "10"),
+	"-d", "-m", file, "59", "58", NULL);
+
+    /* exec failed, unlock the parent anyway! */
+    msg_err("cannot execute trend!");
+    fd = fopen(file, "r");
+    exit(1);
+
+  default:
+    /* feed the pipe */
+    if((fd = fopen(file, "w")))
+    {
+      for(i = 0; i != 59; ++i)
+	fprintf(fd, "%lu\n", wmnd.curdev->his[i][idx]);
+      fclose(fd);
+    }
+  }
+
+  /* remove the fifo */
+  unlink(file);
+}
+#endif
+
+
 void
 click_event(unsigned int region, unsigned int button)
 {
   char* action = NULL;
-
-  if(region == -1u)		/* no region */
+  if(region == REG_NOREG) /* no region */
     return;
 
   msg_dbg(__POSITION__, "clicked btn %d in region %d", button, region);
 
-  if(region == 0)
+  /* the wheel is valid everywhere */
+  if(button == Button4 || button == Button5)
   {
-    switch (button)
+    if(button == Button4)
+      devices_prev();
+    else
+      devices_select(NULL);
+
+    draw_interface();
+  }
+  else if(region == REG_DEV)
+  {
+    switch(button)
     {
     case Button1:
       devices_select(NULL);
       draw_interface();
       break;
+
     case Button3:
       bit_tgl(CFG_SHORTNAME);
       msg_dbg(__POSITION__, "shortname: %d", bit_get(CFG_SHORTNAME));
@@ -535,8 +645,7 @@ click_event(unsigned int region, unsigned int button)
       break;
     }
   }
-  else
-  if(region == 1)
+  else if(region == REG_RT_PB)
   {
     if(button == Button1)
     {
@@ -544,8 +653,7 @@ click_event(unsigned int region, unsigned int button)
       led_control(LED_POWER, bit_get(RUN_ONLINE));
     }
   }
-  else
-  if(region == 2)
+  else if(region == REG_MAIN)
   {
     if(button == Button1)
     {
@@ -559,25 +667,29 @@ click_event(unsigned int region, unsigned int button)
       /* switch time visualization */
       bit_tgl(CFG_SHOWTIME);
   }
-  else
-  if(region == 3)
+  else if(region == REG_SCALE_RX || region == REG_SCALE_TX)
   {
-    switch (button)
+    switch(button)
     {
     case Button1:
       /* switch max screen/history */
       bit_tgl(CFG_MAXSCREEN);
       break;
+#ifdef USE_TREND
+    case Button2:
+      /* launch trend */
+      exec_trend((region == REG_SCALE_RX), bit_get(CFG_MODE));
+      break;
+#endif
     case Button3:
       /* toggle max display */
       bit_tgl(CFG_SHOWMAX);
       break;
     }
   }
-  else
-  if(region == 4)
+  else if(region == REG_SCRIPT)
   {
-    switch (button)
+    switch(button)
     {
     case Button1:
       action = value("bt1_action");
@@ -590,7 +702,7 @@ click_event(unsigned int region, unsigned int button)
       break;
     }
     if(action)
-      execCommand(action);
+      exec_perc_command(action, button);
   }
 }
 
@@ -649,6 +761,7 @@ int main(int argc, char* *argv)
    */
   wmnd.nr_devices = 0;
   wmnd.flags = 0;
+  vars = NULL;
   bit_set(RUN_ONLINE);
   bit_set(CFG_MODE);
 
@@ -657,15 +770,15 @@ int main(int argc, char* *argv)
    * we can use ch, because it's used later
    */
   ch = 0;
-  while (pss_defcon[ch].token)
+  while(pss_defcon[ch].token)
   {
-    assign(pss_defcon[ch].token,pss_defcon[ch].value);
+    assign(pss_defcon[ch].token, pss_defcon[ch].value);
     ch++;
   }
 
   /* parse command line */
   while((ch =
-      getopt(argc, argv, "bc:C:L:d:i:hlmMf:Fr:s:S:tvw:D:I:qQo:n:")) != EOF)
+      getopt(argc, argv, "bc:C:L:d:g:i:hlmMf:Fr:s:S:tvw:D:I:qQo:n:a:")) != EOF)
   {
     switch(ch)
     {
@@ -682,7 +795,10 @@ int main(int argc, char* *argv)
       defcon_touch("md_color", optarg);
       break;
     case 'd':
-      dispname = optarg;
+      defcon_touch("display", optarg);
+      break;
+    case 'g':
+      defcon_touch("geometry", optarg);
       break;
     case 'i':
       defcon_touch("interface_name", optarg);
@@ -739,6 +855,9 @@ int main(int argc, char* *argv)
     case 'n':
       defcon_touch("name", optarg);
       break;
+    case 'a':
+      defcon_touch("fixed_max", optarg);
+      break;
     default:
       usage();
       exit(0);
@@ -763,6 +882,7 @@ int main(int argc, char* *argv)
   wmnd.scroll = MAX(atoi(value("scroll")), 1);
   wmnd.avgSteps = MAX(atoi(value("avg_steps")), 1);
   wmnd.smooth = atof(value("smooth"));
+  wmnd.maxScale = strtol(value("fixed_max"), NULL, 0);
   win_name = value("name");
   if(strval_fe(psi_bool, value("binary_scale")))
     wmnd.scale = binary_scale;
@@ -811,6 +931,7 @@ int main(int argc, char* *argv)
   signal(SIGUSR1, devices_restart);
 
   msg_dbg(__POSITION__, "open X display");
+  dispname = value("display");
   if(!(dockapp.d = XOpenDisplay(dispname)))
   {
     /* fprintf crashes on some systems if dispname is null */
@@ -823,11 +944,12 @@ int main(int argc, char* *argv)
   }
   new_window(win_name, "wmnd", 64, 64, argc, argv);
 
-  add_mr(0, 3, 3, 38, 9);   /* device */
-  add_mr(1, 54, 3, 7, 9);   /* up/down packet/byte mode */
-  add_mr(2, 3, 22, 58, 31); /* main display area */
-  add_mr(3, 3, 13, 58, 9);  /* scale meter */
-  add_mr(4, 3, 54, 58, 7);  /* user script? huh wtf does that do */
+  add_mr(REG_DEV, 3, 3, 38, 9);		/* device */
+  add_mr(REG_RT_PB, 54, 3, 7, 9);	/* up/down packet/byte mode */
+  add_mr(REG_MAIN, 3, 22, 58, 31);	/* main display area */
+  add_mr(REG_SCALE_RX, 3, 13, 29, 9);	/* scale meter, left side (rx) */
+  add_mr(REG_SCALE_TX, 32, 13, 29, 9);	/* scale meter, right side (tx) */
+  add_mr(REG_SCRIPT, 3, 54, 58, 7);	/* user script */
 
   /* updates should begin immediately */
 #ifdef INEXACT_TIMING
@@ -1295,12 +1417,11 @@ draw_max(unsigned long rx, unsigned long tx)
 void
 draw_stats(struct Devices *ptr, const int gap)
 {
-  int bpp; /* bytes/packets per pixel */
   unsigned int k;
   unsigned long* p;
   unsigned int in, out;
   unsigned long rx_max_his, tx_max_his;
-  unsigned long long rx_max, tx_max, max;
+  unsigned long long rx_max, tx_max;
   unsigned int size;
 
   if(bit_get(CFG_SHOWMAX))
@@ -1334,14 +1455,6 @@ draw_stats(struct Devices *ptr, const int gap)
     tx_max = MAX(tx_max, p[out]);
     p += 4;
   }
-  if((max = rx_max + tx_max) > size)
-  {
-    bpp = max / size;
-    if((max % size) > 0)
-      ++bpp;
-  }
-  else
-    bpp = 1;
 
   /* draw rx/tx rate */
   p = ptr->avg;
@@ -1353,7 +1466,7 @@ draw_stats(struct Devices *ptr, const int gap)
     draw_max(rx_max_his, tx_max_his);
 
   p = (unsigned long*)ptr->his;
-  (*drwFuncs[wmnd.wavemode].funcPtr)(p, in, out, size, bpp, rx_max, tx_max);
+  (*drwFuncs[wmnd.wavemode].funcPtr)(p, in, out, size, rx_max, tx_max);
 
   /* copy connection time over the graph */
   if(bit_get(CFG_SHOWTIME) && wmnd.curdev->devstart)
@@ -1511,6 +1624,7 @@ usage(void)
       "  -C <color>          rx color\n"
       "  -L <color>          middle line color\n"
       "  -d <display name>\n"
+      "  -g <geometry>\n"
       "  -h                  this help screen\n"
       "  -i <interface name> select this interface at startup\n"
       "  -l                  start using long device names\n"
@@ -1527,6 +1641,7 @@ usage(void)
       "  -q                  be less verbose\n"
       "  -Q                  enable verbosity\n"
       "  -o <float>          smoothing factor (0-1)\n"
+      "  -a <bps>            fixed max scale at <bps>\n"
       "  -w <mode>           select display mode (see below)\n"
       "  -D <driver>         specify a driver to use\n"
       "  -I <interface name> tell to driver/s the interface to monitor\n\n"
@@ -1672,6 +1787,22 @@ devices_init(const char* driver, const char* interface)
 }
 
 
+void
+devices_prev(void)
+{
+  /*
+   * devices_prev: cycles for the previous device.
+   */
+
+  struct Devices* tmp = devices;
+
+  while(tmp->next && (wmnd.curdev == devices || tmp->next != wmnd.curdev))
+    tmp = tmp->next;
+
+  wmnd.curdev = tmp;
+}
+
+
 void
 devices_select(const char* interface)
 {
@@ -1680,7 +1811,7 @@ devices_select(const char* interface)
    * if name is null, cycles for next device
    */
 
-  struct Devices *prt = devices;
+  struct Devices* prt = devices;
 
   if(interface)
   {
